@@ -18,8 +18,8 @@
 
 static void change_uid(struct process * proc);
 
-void execute_program(struct process * sp, char **argv)
-{
+void
+execute_program(struct process * sp, char **argv) {
 	pid_t pid;
 
 	if (opt_d) {
@@ -28,7 +28,7 @@ void execute_program(struct process * sp, char **argv)
 
 	pid = fork();
 	if (pid<0) {
-		perror("fork");
+		perror("ltrace: fork");
 		exit(1);
 	} else if (!pid) {	/* child */
 		change_uid(sp);
@@ -47,8 +47,8 @@ void execute_program(struct process * sp, char **argv)
 	return;
 }
 
-static void change_uid(struct process * proc)
-{
+static void
+change_uid(struct process * proc) {
 	uid_t run_uid, run_euid;
 	gid_t run_gid, run_egid;
 
@@ -67,7 +67,7 @@ static void change_uid(struct process * proc)
 		run_gid = pent->pw_gid;
 
 		if (initgroups(opt_u, run_gid) < 0) {
-			perror("initgroups");
+			perror("ltrace: initgroups");
 			exit(1);
 		}
 	} else {
@@ -88,11 +88,11 @@ static void change_uid(struct process * proc)
 			}
 		}
 		if (setregid(run_gid, run_egid) < 0) {
-			perror("setregid");
+			perror("ltrace: setregid");
 			exit(1);
 		}
 		if (setreuid(run_uid, run_euid) < 0) {
-			perror("setreuid");
+			perror("ltrace: setreuid");
 			exit(1);
 		}
 	}
