@@ -16,54 +16,17 @@
 #include "read_config_file.h"
 #include "options.h"
 
+#ifndef SYSCONFDIR
+#define SYSCONFDIR "/etc"
+#endif
+
 char * command = NULL;
 struct process * list_of_processes = NULL;
 
 int exiting=0;				/* =1 if a SIGINT or SIGTERM has been received */
 
-static void normal_exit(void);
-static void signal_exit(int);
-static pid_t my_pid;
-
-int main(int argc, char **argv)
-{
-	struct opt_p_t * opt_p_tmp;
-
-	my_pid = getpid();
-	atexit(normal_exit);
-	signal(SIGINT,signal_exit);	/* Detach processes when interrupted */
-	signal(SIGTERM,signal_exit);	/*  ... or killed */
-
-	argv = process_options(argc, argv);
-	read_config_file("/etc/ltrace.conf");
-	if (getenv("HOME")) {
-		char path[PATH_MAX];
-		sprintf(path, getenv("HOME"));	/* FIXME: buffer overrun */
-		strcat(path, "/.ltrace.conf");
-		read_config_file(path);
-	}
-	if (opt_d && opt_e) {
-		struct opt_e_t * tmp = opt_e;
-		while(tmp) {
-			printf("Option -e: %s\n", tmp->name);
-			tmp = tmp->next;
-		}
-	}
-	if (command) {
-		execute_program(open_program(command), argv);
-	}
-	opt_p_tmp = opt_p;
-	while (opt_p_tmp) {
-		open_pid(opt_p_tmp->pid, 1);
-		opt_p_tmp = opt_p_tmp->next;
-	}
-	while(1) {
-		process_event(wait_for_something());
-	}
-}
-
-static void signal_alarm(int sig)
-{
+static void
+signal_alarm(int sig) {
 	struct process * tmp = list_of_processes;
 
 	signal(SIGALRM,SIG_DFL);
@@ -87,8 +50,8 @@ static void signal_alarm(int sig)
 	}
 }
 
-static void signal_exit(int sig)
-{
+static void
+signal_exit(int sig) {
 	exiting=1;
 	if (opt_d) {
 		output_line(0,"Received interrupt signal; exiting...");
@@ -109,7 +72,49 @@ static void signal_exit(int sig)
 	alarm(1);
 }
 
-static void normal_exit(void)
-{
+static void
+normal_exit(void) {
 	output_line(0,0);
 }
+
+int
+main(int argc, char **argv) {
+	struct opt_p_t * opt_p_tmp;
+	char * home;
+
+	atexit(normal_exit);
+	signal(SIGINT,signal_exit);	/* Detach processes when interrupted */
+	signal(SIGTERM,signal_exit);	/*  ... or killed */
+
+	argv = process_options(argc, argv);
+	read_config_file(SYSCONFDIR "/ltrace.conf");
+	home = getenv("HOME");
+	if (home) {
+		char path[PATH_MAX];
+		if (strlen(home) > PATH_MAX-15) {
+			fprintf(stderr, "Error: $HOME too long\n");
+			exit(1);
+		}
+		strcpy(path, getenv("HOME"));
+		strcat(path, "/.ltrace.conf");
+		read_config_file(path);
+	}
+	if (opt_d && opt_e) {
+		struct opt_e_t * tmp = opt_e;
+		while(tmp) {
+			printf("Option -e: %s\n", tmp->name);
+			tmp = tmp->next;
+		}
+	}
+	if (command) {
+		execute_program(open_program(command), argv);
+	}
+	opt_p_tmp = opt_p;
+	while (opt_p_tmp) {
+		open_pid(opt_p_tmp->pid, 1);
+		opt_p_tmp = opt_p_tmp->next;
+	}
+	while(1) {
+		process_event(wait_for_something());
+	}
+}
