@@ -2,6 +2,10 @@
 #include "config.h"
 #endif
 
+#ifndef VERSION
+# define VERSION "0.3.26"
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -17,6 +21,9 @@
 #include "options.h"
 #include "defs.h"
 
+#define MAX_LIBRARY		30
+char *library[MAX_LIBRARY];
+int library_num = 0;
 static char *progname;		/* Program name (`ltrace') */
 FILE * output;
 int opt_a = DEFAULT_ACOLUMN;	/* default alignment column for results */
@@ -32,6 +39,7 @@ int opt_t = 0;			/* print absolute timestamp */
 #if HAVE_LIBIBERTY
 int opt_C = 0;			/* Demangle low-level symbol names into user-level names */
 #endif
+int opt_n = 0;			/* indent trace output according to program flow */
 
 /* List of pids given to option -p: */
 struct opt_p_t * opt_p = NULL;	/* attach to process with a given pid */
@@ -40,8 +48,8 @@ struct opt_p_t * opt_p = NULL;	/* attach to process with a given pid */
 struct opt_e_t * opt_e = NULL;
 int opt_e_enable=1;
 
-static void usage(void)
-{
+static void
+usage(void) {
 #if !(HAVE_GETOPT || HAVE_GETOPT_LONG)
 	fprintf(stdout, "Usage: %s [command [arg ...]]\n"
 "Trace library calls of a given program.\n\n", progname);
@@ -60,6 +68,11 @@ static void usage(void)
 "  -S                  display system calls.\n"
 "  -r                  print relative timestamps.\n"
 "  -t, -tt, -ttt       print absolute timestamps.\n"
+#  if HAVE_GETOPT_LONG
+"  -l, --library=FILE  print library calls from this library only.\n"
+#  else
+"  -l FILE             print library calls from this library only.\n"
+#  endif
 # if HAVE_LIBIBERTY
 #  if HAVE_GETOPT_LONG
 "  -C, --demangle      decode low-level symbol names into user-level names.\n"
@@ -82,6 +95,11 @@ static void usage(void)
 "  -p PID              attach to the process with the process ID pid.\n"
 "  -e expr             modify which events to trace.\n"
 # if HAVE_GETOPT_LONG
+"  -n, --indent=NR     indent output by NR spaces for each call level nesting.\n"
+# else
+"  -n NR               indent output by NR spaces for each call level nesting.\n"
+# endif
+# if HAVE_GETOPT_LONG
 "  -h, --help          display this help and exit.\n"
 # else
 "  -h                  display this help and exit.\n"
@@ -96,9 +114,9 @@ static void usage(void)
 #endif
 }
 
-static char * search_for_command(char * filename)
-{
-	static char pathname[1024];
+static char *
+search_for_command(char * filename) {
+	static char pathname[PATH_MAX];
 	char *path;
 	int m, n;
 
@@ -112,7 +130,11 @@ static char * search_for_command(char * filename)
 		} else {
 			m = n = strlen(path);
 		}
-		strncpy(pathname, path, n);     /* Possible buffer overrun */
+		if (n + strlen(filename) + 1 >= PATH_MAX) {
+			fprintf(stderr, "Error: filename too long\n");
+			exit(1);
+		}
+		strncpy(pathname, path, n);
 		if (n && pathname[n - 1] != '/') {
 			pathname[n++] = '/';
 		}
@@ -124,8 +146,8 @@ static char * search_for_command(char * filename)
 	return filename;
 }
 
-char ** process_options(int argc, char **argv)
-{
+char **
+process_options(int argc, char **argv) {
 	progname = argv[0];
 	output = stderr;
 
@@ -140,6 +162,8 @@ char ** process_options(int argc, char **argv)
 # if HAVE_LIBIBERTY
 			{ "demangle", 0, 0, 'C'},
 #endif
+			{ "indent", 0, 0, 'n'},
+			{ "library", 0, 0, 'l'},
 			{ "help", 0, 0, 'h'},
 			{ "output", 1, 0, 'o'},
 			{ "version", 0, 0, 'V'},
@@ -149,18 +173,24 @@ char ** process_options(int argc, char **argv)
 # if HAVE_LIBIBERTY
 			"C"
 # endif
-			"a:s:o:u:p:e:", long_options, &option_index);
+			"a:s:o:u:p:e:n:l:", long_options, &option_index);
 #else
 		c = getopt(argc, argv, "+dfiLSrthV"
 # if HAVE_LIBIBERTY
 			"C"
 # endif
-			"a:s:o:u:p:e:");
+			"a:s:o:u:p:e:n:l:");
 #endif
 		if (c==-1) {
 			break;
 		}
 		switch(c) {
+			case 'l':	if (library_num == MAX_LIBRARY) {
+						fprintf(stderr, "Too many libraries.  Maximum is %i.\n", MAX_LIBRARY);
+						exit(1);
+					}
+					library[library_num++] = optarg;
+					break;
 			case 'd':	opt_d++;
 					break;
 			case 'f':	opt_f = 1;
@@ -183,6 +213,8 @@ char ** process_options(int argc, char **argv)
 					break;
 			case 's':	opt_s = atoi(optarg);
 					break;
+			case 'n':	opt_n = atoi(optarg);
+					break;
 			case 'h':	usage();
 					exit(0);
 			case 'o':	output = fopen(optarg, "w");
@@ -240,14 +272,8 @@ char ** process_options(int argc, char **argv)
 					}
 					break;
 				}
-			case 'V':	printf("ltrace version "
-#ifdef VERSION
-					VERSION
-#else
-					"???"
-#endif
-					".\n"
-"Copyright (C) 1997-1999 Juan Cespedes <cespedes@debian.org>.\n"
+			case 'V':	printf("ltrace version " VERSION ".\n"
+"Copyright (C) 1997-2002 Juan Cespedes <cespedes@debian.org>.\n"
 "This is free software; see the GNU General Public Licence\n"
 "version 2 or later for copying conditions.  There is NO warranty.\n");
 					exit(0);
