@@ -3,6 +3,7 @@
 #endif
 
 #include <stdio.h>
+#include <sys/types.h>
 
 extern FILE * output;
 extern int opt_a;	/* default alignment column for results */
@@ -18,6 +19,7 @@ extern int opt_t;	/* print absolute timestamp */
 #if HAVE_LIBIBERTY
 extern int opt_C;	/* Demanglelow-level symbol names into user-level names */
 #endif
+extern int opt_n;	/* indent trace output according to program flow */
 
 struct opt_p_t {
 	pid_t pid;
