@@ -7,23 +7,36 @@
 #include <string.h>
 #include <stdlib.h>
 #include <signal.h>
+#include <assert.h>
 
 #include "ltrace.h"
 #include "output.h"
 #include "options.h"
 #include "elf.h"
 
+#ifdef __powerpc__
+#include <sys/ptrace.h>
+#endif
+
 static void process_signal(struct event * event);
 static void process_exit(struct event * event);
 static void process_exit_signal(struct event * event);
 static void process_syscall(struct event * event);
 static void process_sysret(struct event * event);
 static void process_breakpoint(struct event * event);
+static void remove_proc(struct process * proc);
 
-void process_event(struct event * event)
-{
+static void callstack_push_syscall(struct process * proc, int sysnum);
+static void callstack_push_symfunc(struct process * proc, struct library_symbol * sym);
+static void callstack_pop(struct process * proc);
+
+void
+process_event(struct event * event) {
 	switch (event->thing) {
 		case LT_EV_NONE:
+			if (opt_d>0) {
+				output_line(0, "event: none");
+			}
 			return;
 		case LT_EV_SIGNAL:
 			if (opt_d>0) {
@@ -56,6 +69,9 @@ void process_event(struct event * event)
 			process_sysret(event);
 			return;
 		case LT_EV_BREAKPOINT:
+			if (opt_d>0) {
+				output_line(0, "event: breakpoint");
+			}
 			process_breakpoint(event);
 			return;
 		default:
@@ -64,22 +80,22 @@ void process_event(struct event * event)
 	}
 }
 
-static char * shortsignal(int signum)
-{
+static char *
+shortsignal(int signum) {
 	static char * signalent0[] = {
 	#include "signalent.h"
 	};
 	int nsignals0 = sizeof signalent0 / sizeof signalent0[0];
 
-	if (signum<0 || signum>nsignals0) {
+	if (signum<0 || signum>=nsignals0) {
 		return "UNKNOWN_SIGNAL";
 	} else {
 		return signalent0[signum];
 	}
 }
 
-static char * sysname(int sysnum)
-{
+static char *
+sysname(int sysnum) {
 	static char result[128];
 	static char * syscalent0[] = {
 	#include "syscallent.h"
@@ -95,10 +111,8 @@ static char * sysname(int sysnum)
 	}
 }
 
-static void remove_proc(struct process * proc);
-
-static void process_signal(struct event * event)
-{
+static void
+process_signal(struct event * event) {
 	if (exiting && event->e_un.signum == SIGSTOP) {
 		pid_t pid = event->proc->pid;
 		disable_all_breakpoints(event->proc);
@@ -112,22 +126,22 @@ static void process_signal(struct event * event)
 	continue_after_signal(event->proc->pid, event->e_un.signum);
 }
 
-static void process_exit(struct event * event)
-{
+static void
+process_exit(struct event * event) {
 	output_line(event->proc, "+++ exited (status %d) +++",
 		event->e_un.ret_val);
 	remove_proc(event->proc);
 }
 
-static void process_exit_signal(struct event * event)
-{
+static void
+process_exit_signal(struct event * event) {
 	output_line(event->proc, "+++ killed by %s +++",
 		shortsignal(event->e_un.signum));
 	remove_proc(event->proc);
 }
 
-static void remove_proc(struct process * proc)
-{
+static void
+remove_proc(struct process * proc) {
 	struct process *tmp, *tmp2;
 
 	if (opt_d) {
@@ -152,12 +166,12 @@ static void remove_proc(struct process * proc)
 	}
 }
 
-static void process_syscall(struct event * event)
-{
-	event->proc->current_syscall = event->e_un.sysnum;
+static void
+process_syscall(struct event * event) {
 	if (opt_S) {
 		output_left(LT_TOF_SYSCALL, event->proc, sysname(event->e_un.sysnum));
 	}
+	callstack_push_syscall(event->proc, event->e_un.sysnum);
 	if (fork_p(event->e_un.sysnum) || exec_p(event->e_un.sysnum)) {
 		disable_all_breakpoints(event->proc);
 	} else if (!event->proc->breakpoints_enabled) {
@@ -166,18 +180,18 @@ static void process_syscall(struct event * event)
 	continue_process(event->proc->pid);
 }
 
-static void process_sysret(struct event * event)
-{
-	if (opt_S) {
-		output_right(LT_TOF_SYSCALL, event->proc, sysname(event->e_un.sysnum));
-	}
+static void
+process_sysret(struct event * event) {
 	if (exec_p(event->e_un.sysnum)) {
 		if (gimme_arg(LT_TOF_SYSCALL,event->proc,-1)==0) {
+
 			event->proc->filename = pid2name(event->proc->pid);
 			event->proc->list_of_symbols = read_elf(event->proc->filename);
+
+			/* The kernel will stop the process just after an execve()
+			 * and we will be able to enable breakpoints again
+			 */
 			event->proc->breakpoints_enabled = -1;
-		} else {
-			enable_all_breakpoints(event->proc);
 		}
 	}
 	if (fork_p(event->e_un.sysnum)) {
@@ -189,42 +203,68 @@ static void process_sysret(struct event * event)
 		}
 		enable_all_breakpoints(event->proc);
 	}
-	event->proc->current_syscall = -1;
+	callstack_pop(event->proc);
+	if (opt_S) {
+		output_right(LT_TOF_SYSCALL, event->proc, sysname(event->e_un.sysnum));
+	}
 	continue_process(event->proc->pid);
 }
 
-static void process_breakpoint(struct event * event)
-{
+static void
+process_breakpoint(struct event * event) {
 	struct library_symbol * tmp;
+	int i,j;
 
 	if (opt_d>1) {
 		output_line(0,"event: breakpoint (0x%08x)", event->e_un.brk_addr);
 	}
 	if (event->proc->breakpoint_being_enabled) {
+		/* Reinsert breakpoint */
 		continue_enabling_breakpoint(event->proc->pid, event->proc->breakpoint_being_enabled);
 		event->proc->breakpoint_being_enabled = NULL;
 		return;
 	}
-	if (event->proc->current_symbol && event->e_un.brk_addr == event->proc->return_value.addr) {
-		output_right(LT_TOF_FUNCTION, event->proc, event->proc->current_symbol->name);
-		continue_after_breakpoint(event->proc, &event->proc->return_value, 1);
-		event->proc->current_symbol = NULL;
-		return;
+
+	for(i=event->proc->callstack_depth-1; i>=0; i--) {
+		if (event->e_un.brk_addr == event->proc->callstack[i].return_addr) {
+#ifdef __powerpc__
+                       unsigned long a;
+                       unsigned long addr = event->proc->callstack[i].c_un.libfunc->enter_addr;
+                       struct breakpoint *sbp = dict_find_entry(event->proc, addr);
+                       unsigned char break_insn[] = BREAKPOINT_VALUE;
+
+                       /*
+                        * PPC HACK! (XXX FIXME TODO)
+                        * The PLT gets modified during the first call,
+                        * so be sure to re-enable the breakpoint.
+                        */
+                       a = ptrace(PTRACE_PEEKTEXT, event->proc->pid, addr);
+
+                       if (memcmp(&a, break_insn, 4)) {
+                               sbp->enabled--;
+                               insert_breakpoint(event->proc, addr);
+                       }
+#endif
+			for(j=event->proc->callstack_depth-1; j>=i; j--) {
+				callstack_pop(event->proc);
+			}
+			event->proc->return_addr = event->e_un.brk_addr;
+			output_right(LT_TOF_FUNCTION, event->proc,
+					event->proc->callstack[i].c_un.libfunc->name);
+			continue_after_breakpoint(event->proc,
+					address2bpstruct(event->proc, event->e_un.brk_addr));
+			return;
+		}
 	}
 
 	tmp = event->proc->list_of_symbols;
 	while(tmp) {
-		if (event->e_un.brk_addr == tmp->brk.addr) {
-			if (event->proc->current_symbol) {
-				delete_breakpoint(event->proc->pid, &event->proc->return_value);
-			}
-			event->proc->current_symbol = tmp;
+		if (event->e_un.brk_addr == tmp->enter_addr) {
 			event->proc->stack_pointer = get_stack_pointer(event->proc->pid);
 			event->proc->return_addr = get_return_addr(event->proc->pid, event->proc->stack_pointer);
 			output_left(LT_TOF_FUNCTION, event->proc, tmp->name);
-			event->proc->return_value.addr = event->proc->return_addr;
-			insert_breakpoint(event->proc->pid, &event->proc->return_value);
-			continue_after_breakpoint(event->proc, &tmp->brk, 0);
+			callstack_push_symfunc(event->proc, tmp);
+			continue_after_breakpoint(event->proc, address2bpstruct(event->proc, tmp->enter_addr));
 			return;
 		}
 		tmp = tmp->next;
@@ -233,3 +273,53 @@ static void process_breakpoint(struct event * event)
 		(unsigned)event->e_un.brk_addr);
 	continue_process(event->proc->pid);
 }
+
+static void
+callstack_push_syscall(struct process * proc, int sysnum) {
+	struct callstack_element * elem;
+
+	/* FIXME: not good -- should use dynamic allocation. 19990703 mortene. */
+	if (proc->callstack_depth == MAX_CALLDEPTH-1) {
+		fprintf(stderr, "Error: call nesting too deep!\n");
+		return;
+	}
+
+	elem = & proc->callstack[proc->callstack_depth];
+	elem->is_syscall = 1;
+	elem->c_un.syscall = sysnum;
+	elem->return_addr = NULL;
+
+	proc->callstack_depth++;
+}
+
+static void
+callstack_push_symfunc(struct process * proc, struct library_symbol * sym) {
+	struct callstack_element * elem;
+
+	/* FIXME: not good -- should use dynamic allocation. 19990703 mortene. */
+	if (proc->callstack_depth == MAX_CALLDEPTH-1) {
+		fprintf(stderr, "Error: call nesting too deep!\n");
+		return;
+	}
+
+	elem = & proc->callstack[proc->callstack_depth];
+	elem->is_syscall = 0;
+	elem->c_un.libfunc = sym;
+
+	elem->return_addr = proc->return_addr;
+	insert_breakpoint(proc, elem->return_addr);
+
+	proc->callstack_depth++;
+}
+
+static void
+callstack_pop(struct process * proc) {
+	struct callstack_element * elem;
+	assert(proc->callstack_depth > 0);
+
+	elem = & proc->callstack[proc->callstack_depth-1];
+	if (!elem->is_syscall) {
+		delete_breakpoint(proc, elem->return_addr);
+	}
+	proc->callstack_depth--;
+}
