@@ -44,8 +44,8 @@ static struct list_of_pt_t {
 	{ NULL,     ARGTYPE_UNKNOWN }		/* Must finish with NULL */
 };
 
-static enum arg_type str2type(char ** str)
-{
+static enum arg_type
+str2type(char ** str) {
 	struct list_of_pt_t * tmp = &list_of_pt[0];
 
 	while(tmp->name) {
@@ -59,8 +59,8 @@ static enum arg_type str2type(char ** str)
 	return ARGTYPE_UNKNOWN;
 }
 
-static void eat_spaces(char ** str)
-{
+static void
+eat_spaces(char ** str) {
 	while(**str==' ') {
 		(*str)++;
 	}
@@ -69,11 +69,9 @@ static void eat_spaces(char ** str)
 /*
   Returns position in string at the left parenthesis which starts the
   function's argument signature. Returns NULL on error.
-  FIXME: this routine incorrectly returns the beginning of the string in
-         things like "hello)" (cespedes)
 */
-static char * start_of_arg_sig(char * str)
-{
+static char *
+start_of_arg_sig(char * str) {
 	char * pos;
 	int stacked = 0;
 
@@ -82,11 +80,12 @@ static char * start_of_arg_sig(char * str)
 	pos = &str[strlen(str)];
 	do {
 		pos--;
+		if (pos < str) return NULL;
 		while ((pos > str) && (*pos != ')') && (*pos != '(')) pos--;
 
-		if (pos < str) return NULL;
-		else if (*pos == ')') stacked++;
-		else stacked--;
+		if (*pos == ')') stacked++;
+		else if (*pos == '(') stacked--;
+		else return NULL;
 
 	} while (stacked > 0);
 
@@ -96,7 +95,8 @@ static char * start_of_arg_sig(char * str)
 static int line_no;
 static char * filename;
 
-struct function * process_line (char * buf) {
+struct function *
+process_line (char * buf) {
 	struct function fun;
 	struct function * fun_p;
 	char * str = buf;
@@ -119,12 +119,7 @@ struct function * process_line (char * buf) {
 		output_line(0, " return_type = %d", fun.return_type);
 	}
 	eat_spaces(&str);
-#if 0  /* Obsoleted code. 19990702 mortene. */
-       /* FIXME: why the space? 19990702 mortene. */
-	tmp = strpbrk(str, " (");
-#else
 	tmp = start_of_arg_sig(str);
-#endif
 	if (!tmp) {
 		output_line(0, "Syntax error in `%s', line %d", filename, line_no);
 		return NULL;
@@ -169,8 +164,8 @@ struct function * process_line (char * buf) {
 	return fun_p;
 }
 
-void read_config_file(char * file)
-{
+void
+read_config_file(char * file) {
 	FILE * stream;
 	char buf[1024];
 
