@@ -14,18 +14,22 @@
 # define PTRACE_POKEUSER PTRACE_POKEUSR
 #endif
 
-void * get_instruction_pointer(pid_t pid)
-{
+void *
+get_instruction_pointer(pid_t pid) {
 	return (void *)ptrace(PTRACE_PEEKUSER, pid, 4*PT_PC, 0);
 }
 
-void * get_stack_pointer(pid_t pid)
-{
+void
+set_instruction_pointer(pid_t pid, long addr) {
+	ptrace(PTRACE_POKEUSER, pid, 4*PT_PC, addr);
+}
+
+void *
+get_stack_pointer(pid_t pid) {
 	return (void *)ptrace(PTRACE_PEEKUSER, pid, 4*PT_USP, 0);
 }
 
-void * get_return_addr(pid_t pid, void * stack_pointer)
-{
+void *
+get_return_addr(pid_t pid, void * stack_pointer) {
 	return (void *)ptrace(PTRACE_PEEKTEXT, pid, stack_pointer, 0);
 }
-
