@@ -3,6 +3,7 @@
 #endif
 
 #define	_GNU_SOURCE	1
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <errno.h>
@@ -19,8 +20,8 @@ static struct event event;
 
 static struct process * pid2proc(int pid);
 
-struct event * wait_for_something(void)
-{
+struct event *
+wait_for_something(void) {
 	pid_t pid;
 	int status;
 	int tmp;
@@ -73,7 +74,6 @@ struct event * wait_for_something(void)
 		case 2:	event.thing = LT_EV_SYSRET;
 			event.e_un.sysnum = tmp;
 			return &event;
-		default:
 	}
 	if (WIFEXITED(status)) {
 		event.thing = LT_EV_EXIT;
@@ -102,8 +102,8 @@ struct event * wait_for_something(void)
 	return &event;
 }
 
-static struct process * pid2proc(pid_t pid)
-{
+static struct process *
+pid2proc(pid_t pid) {
 	struct process * tmp;
 
 	tmp = list_of_processes;
@@ -115,4 +115,3 @@ static struct process * pid2proc(pid_t pid)
 	}
 	return NULL;
 }
-
